// outputs a list of differing bits
// usage: bit_diff <file1> <file2>

// won't work for files/frames larger than a megabyte

#include <iostream>
#include <ostream>
#include <fstream>

#include <boost/lexical_cast.hpp>

int main(int argc, char * argv[]) {
	if (argc != 3)
		exit(1);

	// read
	std::ifstream infile1(argv[1]);
	std::ifstream infile2(argv[2]);

	const int bufsize = 1048576;
	char buf1[bufsize], buf2[bufsize];

	infile1.read(&buf1[0], bufsize);
	infile2.read(&buf2[0], bufsize);

	int num_read = infile1.gcount();
	if (infile2.gcount() != num_read) {
		std::cerr << "Files differ in size: " << num_read << " vs. " << infile2.gcount() << ", aborting comparison.\n";
		exit(0);
	}

	// compare
	for (int i = 0; i < num_read; ++i) {
		for (int b = 7; b >= 0; --b) {
			if (((buf1[i] >> b) & 0x1) != ((buf2[i] >> b) & 0x1)) {
				std::cout << (i * 8 + (7 - b)) << "\n";
			}
		}
	}

	return 0;
}

