// outputs an image with XOR of bytes of files represented as grayscale pixels
// usage: byte_diff_image <file1> <file2> <outfile>

// won't work for files/frames larger than a megabyte

#include <iostream>
#include <ostream>
#include <fstream>

#include <boost/lexical_cast.hpp>

#include <Imlib2.h>

int main(int argc, char * argv[]) {
	if (argc != 4)
		exit(1);

	// read
	std::ifstream infile1(argv[1]);
	std::ifstream infile2(argv[2]);

	const int bufsize = 1048576;
	char buf1[bufsize], buf2[bufsize];

	infile1.read(&buf1[0], bufsize);
	infile2.read(&buf2[0], bufsize);

	int num_read = infile1.gcount();
	if (infile2.gcount() != num_read) {
		std::cerr << "Files differ in size: " << num_read << " vs. " << infile2.gcount() << ", aborting comparison.\n";
		exit(0);
	}

	// compare and create image
	DATA32 outbuf[bufsize];
	for (int i = 0; i < num_read; ++i) {
		unsigned int var = reinterpret_cast<unsigned char *>(buf1)[i] ^ reinterpret_cast<unsigned char *>(buf2)[i];
		outbuf[i] = (var << 16) | (var << 8) | var;
		if (var != 0)
			std::cerr << "var: " << var << ", outbuf[i]: " << outbuf[i] << "\n";
	}

	// save image
	Imlib_Image image = imlib_create_image_using_data(28, (num_read + 27) / 28, &outbuf[0]);
	imlib_context_set_image(image);
	imlib_save_image(argv[3]);
	imlib_free_image();

	return 0;
}

