// compute the sum of color variances across a specific block
// read PNG from standard input

#include <iostream>
#include <ostream>

#include <boost/lexical_cast.hpp>

#include <Imlib2.h>


double calc_col_var(DATA32 d) {
	double col_mean = 0.0;
	for (int i = 0; i < 3; ++i)
		col_mean += (d >> (i*8)) & 0xff;
	col_mean /= 3.0;

	double col_var = 0.0;
	for (int i = 0; i < 3; ++i) {
		double diff = ((d >> (i*8)) & 0xff) - col_mean;
		col_var += diff * diff;
	}
	col_var /= 3.0;
	return col_var;
}

// arguments: mb_col mb_row [id]
int main(int argc, char *argv[]) {
	if (argc < 3) exit(1);
	int mb_col = boost::lexical_cast<int>(argv[1]);
	int mb_row = boost::lexical_cast<int>(argv[2]);

	// load image
	Imlib_Image image = imlib_load_image("/dev/stdin");
	imlib_context_set_image(image);
	int width = imlib_image_get_width();
	int height = imlib_image_get_height();
	DATA32 * data = imlib_image_get_data_for_reading_only();

	double scv = 0.0;
	for (int x = mb_col * 16; x < mb_col * 16 + 16; ++x)
		for (int y = mb_row * 16; y < mb_row * 16 + 16; ++y)
			scv += calc_col_var(data[y * width + x]);

	std::cout.precision(15);
	std::cout.fill('0');
	std::cout.width(15);
	std::cout << scv;
	if (argc > 3)
		std::cout << " " << argv[3];
	std::cout << "\n";

	imlib_context_set_image(image);
	imlib_free_image();

	return 0;
}



