// compute the sum of squared errors across all macroblock edges
// read PNG from standard input

#include <iostream>
#include <ostream>

#include <Imlib2.h>


double calc_sse(DATA32 d1, DATA32 d2) {
	double res = 0.0;
	for (int i = 0; i < 3; ++i) {
		int v1 = (d1 >> (i*8)) & 0xff;
		int v2 = (d2 >> (i*8)) & 0xff;
		double diff = v2 - v1;
		res += diff * diff;
	}
	return res;
}

int main(int argc, char *argv[]) {
	// load image
	Imlib_Image image = imlib_load_image("/dev/stdin");
	imlib_context_set_image(image);
	int width = imlib_image_get_width();
	int height = imlib_image_get_height();
	DATA32 * data = imlib_image_get_data_for_reading_only();

	double sse = 0.0;
	for (int x = 16; x < width; x += 16)
		for (int y = 0; y < height; ++y)
			sse += calc_sse(data[y * width + x], data[y * width + x - 1]);

	for (int y = 16; y < height; y += 16)
		for (int x = 0; x < width; ++x)
			sse += calc_sse(data[y * width + x], data[y * width + x - width]);

	std::cout.precision(15);
	std::cout.fill('0');
	std::cout.width(15);
	std::cout << sse;
	if (argc > 1)
		std::cout << " " << argv[1];
	std::cout << "\n";

	imlib_context_set_image(image);
	imlib_free_image();

	return 0;
}



