// find the first macroblock (in left-to-right, top-to-bottom order)
// whose colour variance along left and top edges or internal block edges is too large

// read PNG from file given on command line, or stdin if not given

#include <cmath>
#include <iostream>
#include <ostream>
#include <vector>

#include <Imlib2.h>

double calc_sse(DATA32 d1, DATA32 d2) {
	double res = 0.0;
	for (int i = 0; i < 3; ++i) {
		int v1 = (d1 >> (i*8)) & 0xff;
		int v2 = (d2 >> (i*8)) & 0xff;
		double diff = v2 - v1;
		res += diff * diff;
	}
	return res;
}


// <name> file_to_read.png
int main(int argc, char *argv[]) {
	std::string filename("/dev/stdin");
	if (argc > 1)
		filename = argv[1];

	// load image
	Imlib_Image image = imlib_load_image(filename.c_str());
	imlib_context_set_image(image);
	int width = imlib_image_get_width();
	int height = imlib_image_get_height();
	DATA32 * data = imlib_image_get_data_for_reading_only();

	const int threshold = 10000;

	int row = 0, col = 0;
	double sse = 0.0, sumsse = 0.0;
	while ((row < 30) && (col < 44)) {
		sumsse += sse;
		sse = 0.0;
		int x, y;
		// internal vertical edge
/*		x = col * 16 + 8;
		for (int y = row * 16; y < row * 16 + 16; ++y)
			sse += calc_sse(data[y * width + x], data[y * width + x - 1]);
*/
		// left edge
		if (col > 0) {
			x = col * 16;
			for (int y = row * 16; y < row * 16 + 16; ++y)
				sse += calc_sse(data[y * width + x], data[y * width + x - 1]);
		}

		// internal horizontal edge
/*		y = row * 16 + 8;
		for (int x = col * 16; x < col * 16 + 16; ++x)
			sse += calc_sse(data[y * width + x], data[y * width + x - width]);
*/
		// top edge
		if (row > 0) {
			y = row * 16;
			for (int x = col * 16; x < col * 16 + 16; ++x)
			sse += calc_sse(data[y * width + x], data[y * width + x - width]);
		}

		std::cout << col << "," << row << ": " << sse << "\n";

		if (sse > threshold) break;

		// next
		++col;
		if (col == 44) {
			col = 0;
			++row;
		}
	}
	// Bash can't deal with floating point numbers, so make it a big integer instead
	int score = round((row * 44.0 + col + (1.0 - 1.0 / log(sumsse + sse))) * 1000.0);

	std::cout << score;
	std::cout << "\n";

	imlib_context_set_image(image);
	imlib_free_image();

	return 0;
}



