// compute the sum of squared errors across the top and left edge of a specific macro block
// also compute across the edges that run through the middle of the MB and separate the blocks
// read PNG from standard input

#include <iostream>
#include <ostream>

#include <boost/lexical_cast.hpp>

#include <Imlib2.h>


double calc_sse(DATA32 d1, DATA32 d2) {
	double res = 0.0;
	for (int i = 0; i < 3; ++i) {
		int v1 = (d1 >> (i*8)) & 0xff;
		int v2 = (d2 >> (i*8)) & 0xff;
		double diff = v2 - v1;
		res += diff * diff;
	}
	return res;
}

// arguments: mb_col mb_row [id]
int main(int argc, char *argv[]) {
	if (argc < 3) exit(1);
	int mb_col = boost::lexical_cast<int>(argv[1]);
	int mb_row = boost::lexical_cast<int>(argv[2]);

	// load image
	Imlib_Image image = imlib_load_image("/dev/stdin");
	imlib_context_set_image(image);
	int width = imlib_image_get_width();
	int height = imlib_image_get_height();
	DATA32 * data = imlib_image_get_data_for_reading_only();

	double sse = 0.0;
	int x = mb_col * 16 + 8;
	for (int y = mb_row * 16; y < mb_row * 16 + 16; ++y)
		sse += calc_sse(data[y * width + x], data[y * width + x - 1]);

	if (mb_col > 0) {
		x = mb_col * 16;
		for (int y = mb_row * 16; y < mb_row * 16 + 16; ++y)
			sse += calc_sse(data[y * width + x], data[y * width + x - 1]);
	}

	int y = mb_row * 16 + 8;
	for (int x = mb_col * 16; x < mb_col * 16 + 16; ++x)
		sse += calc_sse(data[y * width + x], data[y * width + x - width]);

	if (mb_row > 0) {
		y = mb_row * 16;
		for (int x = mb_col * 16; x < mb_col * 16 + 16; ++x)
			sse += calc_sse(data[y * width + x], data[y * width + x - width]);
	}

	std::cout.precision(15);
	std::cout.fill('0');
	std::cout.width(15);
	std::cout << sse;
	if (argc > 3)
		std::cout << " " << argv[3];
	std::cout << "\n";

	imlib_context_set_image(image);
	imlib_free_image();

	return 0;
}



